@@ -6,9 +6,8 @@ from tryton.config import CONFIG
 import gtk
 import pango
 import gettext
-from tryton.exceptions import TrytonServerError
 from tryton.gui.window.nomodal import NoModal
-import tryton.rpc as rpc
+from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
@@ -17,69 +16,77 @@ class WinForm(NoModal):
     "Form window"
 
     def __init__(self, screen, callback, view_type='form',
-            new=False, many=False, context=None):
+            new=False, many=0, domain=None, context=None,
+            save_current=False):
         NoModal.__init__(self)
         self.screen = screen
         self.callback = callback
+        self.many = many
+        self.domain = domain
         self.context = context
+        self.save_current = save_current
         self.prev_view = self.screen.current_view
         self.screen.screen_container.alternate_view = True
-        switch_new = False
-        if view_type == 'form' and not self.screen.current_record:
-            switch_new = True
         if view_type not in (x.view_type for x in self.screen.views) and \
                 view_type not in self.screen.view_to_load:
-            self.screen.add_view_id(False, view_type, display=True)
-        else:
-            self.screen.switch_view(view_type=view_type, context=context)
-        if new and not switch_new:
-            self.screen.new(context=self.context)
+            self.screen.add_view_id(None, view_type)
+        self.screen.switch_view(view_type=view_type)
+        if new:
+            self.screen.new()
         self.win = gtk.Dialog(_('Link'), self.parent,
                 gtk.DIALOG_DESTROY_WITH_PARENT)
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.win.set_icon(TRYTON_ICON)
         self.win.set_has_separator(False)
+        self.win.set_deletable(False)
+        self.win.connect('close', self.close)
         self.win.connect('response', self.response)
 
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
-        self.but_cancel = None
+        readonly = self.screen.readonly or self.screen.group.readonly
+
         self.but_ok = None
         self.but_new = None
-        if new:
-            icon_cancel = gtk.STOCK_CANCEL
-            self.but_cancel = self.win.add_button(icon_cancel,
-                    gtk.RESPONSE_CANCEL)
 
-        self.but_ok = self.win.add_button(gtk.STOCK_OK,
-            gtk.RESPONSE_OK)
-        if new and many:
-            self.but_ok.add_accelerator('clicked',
-                self.accel_group, gtk.keysyms.Return,
-                gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK,
-                gtk.ACCEL_VISIBLE)
+        if view_type == 'form':
+            if not new and self.screen.current_record.id < 0:
+                stock_id = gtk.STOCK_DELETE
+            else:
+                stock_id = gtk.STOCK_CANCEL
+            self.but_cancel = self.win.add_button(stock_id,
+                gtk.RESPONSE_CANCEL)
 
+        if new and self.many:
             self.but_new = self.win.add_button(gtk.STOCK_NEW,
                 gtk.RESPONSE_ACCEPT)
-            self.but_new.add_accelerator('clicked', self.accel_group,
-                gtk.keysyms.Return, gtk.gdk.CONTROL_MASK,
-                gtk.ACCEL_VISIBLE)
-            self.win.set_default_response(gtk.RESPONSE_ACCEPT)
+            self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
+
+        if self.save_current:
+            self.but_ok = gtk.Button(_('_Save'))
+            img_save = gtk.Image()
+            img_save.set_from_stock('tryton-save', gtk.ICON_SIZE_BUTTON)
+            self.but_ok.set_image(img_save)
+            self.but_ok.set_accel_path('<tryton>/Form/Save', self.accel_group)
+            self.but_ok.set_can_default(True)
+            self.but_ok.show()
+            self.win.add_action_widget(self.but_ok, gtk.RESPONSE_OK)
         else:
-            self.but_ok.add_accelerator('clicked', self.accel_group,
-                gtk.keysyms.Return, gtk.gdk.CONTROL_MASK,
-                gtk.ACCEL_VISIBLE)
-            self.win.set_default_response(gtk.RESPONSE_OK)
+            self.but_ok = self.win.add_button(gtk.STOCK_OK,
+                gtk.RESPONSE_OK)
+        self.but_ok.add_accelerator('clicked', self.accel_group,
+            gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+        self.win.set_default_response(gtk.RESPONSE_OK)
 
         self.win.set_title(self.screen.current_view.title)
 
         title = gtk.Label()
-        title.set_use_markup(True)
-        title.modify_font(pango.FontDescription("12"))
-        title.set_label('<b>' + self.screen.current_view.title + '</b>')
+        title.modify_font(pango.FontDescription("bold 12"))
+        title.set_label(self.screen.current_view.title)
         title.set_padding(20, 3)
         title.set_alignment(0.0, 0.5)
+        title.set_size_request(0, -1)  # Allow overflow
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
         title.show()
 
@@ -116,9 +123,45 @@ class WinForm(NoModal):
         if view_type == 'tree':
             hbox = gtk.HBox(homogeneous=False, spacing=0)
             tooltips = common.Tooltips()
+            access = common.MODELACCESS[screen.model_name]
+
+            if domain is not None:
+                self.wid_text = gtk.Entry()
+                self.wid_text.set_property('width_chars', 13)
+                self.wid_text.connect('activate', self._sig_activate)
+                self.wid_text.connect('focus-out-event', self._focus_out)
+                hbox.pack_start(self.wid_text, expand=True, fill=True)
+
+                self.but_add = gtk.Button()
+                tooltips.set_tip(self.but_add, _('Add'))
+                self.but_add.connect('clicked', self._sig_add)
+                img_add = gtk.Image()
+                img_add.set_from_stock('tryton-list-add',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
+                img_add.set_alignment(0.5, 0.5)
+                self.but_add.add(img_add)
+                self.but_add.set_relief(gtk.RELIEF_NONE)
+                hbox.pack_start(self.but_add, expand=False, fill=False)
+                if not access['read'] or readonly:
+                    self.but_add.set_sensitive(False)
+
+                self.but_remove = gtk.Button()
+                tooltips.set_tip(self.but_remove, _('Remove <Del>'))
+                self.but_remove.connect('clicked', self._sig_remove, True)
+                img_remove = gtk.Image()
+                img_remove.set_from_stock('tryton-list-remove',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
+                img_remove.set_alignment(0.5, 0.5)
+                self.but_remove.add(img_remove)
+                self.but_remove.set_relief(gtk.RELIEF_NONE)
+                hbox.pack_start(self.but_remove, expand=False, fill=False)
+                if not access['read'] or readonly:
+                    self.but_remove.set_sensitive(False)
+
+                hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
             self.but_new = gtk.Button()
-            tooltips.set_tip(self.but_new, _('Create a new record'))
+            tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
             self.but_new.connect('clicked', self._sig_new)
             img_new = gtk.Image()
             img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -126,10 +169,12 @@ class WinForm(NoModal):
             self.but_new.add(img_new)
             self.but_new.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_new, expand=False, fill=False)
+            if not access['create'] or readonly:
+                self.but_new.set_sensitive(False)
 
             self.but_del = gtk.Button()
-            tooltips.set_tip(self.but_del, _('Delete selected record'))
-            self.but_del.connect('clicked', self._sig_remove)
+            tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
+            self.but_del.connect('clicked', self._sig_remove, False)
             img_del = gtk.Image()
             img_del.set_from_stock('tryton-delete',
                 gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -137,33 +182,49 @@ class WinForm(NoModal):
             self.but_del.add(img_del)
             self.but_del.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_del, expand=False, fill=False)
+            if not access['delete'] or readonly:
+                self.but_del.set_sensitive(False)
+
+            self.but_undel = gtk.Button()
+            tooltips.set_tip(self.but_undel,
+                _('Undelete selected record <Ins>'))
+            self.but_undel.connect('clicked', self._sig_undelete)
+            img_undel = gtk.Image()
+            img_undel.set_from_stock('tryton-undo',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_undel.set_alignment(0.5, 0.5)
+            self.but_undel.add(img_undel)
+            self.but_undel.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(self.but_undel, expand=False, fill=False)
+            if not access['delete'] or readonly:
+                self.but_undel.set_sensitive(False)
 
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
-            but_pre = gtk.Button()
-            tooltips.set_tip(but_pre, _('Previous'))
-            but_pre.connect('clicked', self._sig_previous)
+            self.but_pre = gtk.Button()
+            tooltips.set_tip(self.but_pre, _('Previous'))
+            self.but_pre.connect('clicked', self._sig_previous)
             img_pre = gtk.Image()
             img_pre.set_from_stock('tryton-go-previous',
                 gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_pre.set_alignment(0.5, 0.5)
-            but_pre.add(img_pre)
-            but_pre.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(but_pre, expand=False, fill=False)
+            self.but_pre.add(img_pre)
+            self.but_pre.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(self.but_pre, expand=False, fill=False)
 
             self.label = gtk.Label('(0,0)')
             hbox.pack_start(self.label, expand=False, fill=False)
 
-            but_next = gtk.Button()
-            tooltips.set_tip(but_next, _('Next'))
-            but_next.connect('clicked', self._sig_next)
+            self.but_next = gtk.Button()
+            tooltips.set_tip(self.but_next, _('Next'))
+            self.but_next.connect('clicked', self._sig_next)
             img_next = gtk.Image()
             img_next.set_from_stock('tryton-go-next',
                 gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_next.set_alignment(0.5, 0.5)
-            but_next.add(img_next)
-            but_next.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(but_next, expand=False, fill=False)
+            self.but_next.add(img_next)
+            self.but_next.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(self.but_next, expand=False, fill=False)
 
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
@@ -178,6 +239,8 @@ class WinForm(NoModal):
             but_switch.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(but_switch, expand=False, fill=False)
 
+            but_switch.props.sensitive = screen.number_of_views > 1
+
             tooltips.enable()
 
             alignment = gtk.Alignment(1.0)
@@ -204,6 +267,12 @@ class WinForm(NoModal):
             self.screen.screen_container.alternate_viewport.connect(
                     'key-press-event', self.on_keypress)
 
+        if self.save_current:
+            self.screen.signal_connect(self, 'record-message',
+                self.activate_save)
+            self.screen.signal_connect(self, 'record-modified',
+                self.activate_save)
+
         self.register()
         self.win.show()
 
@@ -236,7 +305,7 @@ class WinForm(NoModal):
         self.screen.switch_view()
 
     def _sig_new(self, widget):
-        self.screen.new(context=self.context)
+        self.screen.new()
         self.screen.current_view.widget.set_sensitive(True)
 
     def _sig_next(self, widget):
@@ -245,17 +314,23 @@ class WinForm(NoModal):
     def _sig_previous(self, widget):
         self.screen.display_prev()
 
-    def _sig_remove(self, widget):
-        self.screen.remove(delete=True)
+    def _sig_remove(self, widget, remove=False):
+        self.screen.remove(remove=remove)
+
+    def _sig_undelete(self, button):
+        self.screen.unremove()
 
     def _sig_activate(self, *args):
         self._sig_add()
         self.wid_text.grab_focus()
 
+    def _focus_out(self, *args):
+        if self.wid_text.get_text():
+            self._sig_add()
+
     def _sig_add(self, *args):
         from tryton.gui.window.win_search import WinSearch
-        domain = []
-        context = rpc.CONTEXT.copy()
+        domain = self.domain[:]
         model_name = self.screen.model_name
 
         try:
@@ -264,64 +339,120 @@ class WinForm(NoModal):
                         '%' + self.wid_text.get_text() + '%'), domain]
             else:
                 dom = domain
-            ids = rpc.execute('model', model_name, 'search', dom,
-                    0, CONFIG['client.limit'], None, context)
-        except TrytonServerError, exception:
-            common.process_exception(exception)
+            ids = RPCExecute('model', model_name, 'search', dom,
+                    0, CONFIG['client.limit'], None, context=self.context)
+        except RPCException:
             return False
 
-        def callback(ids):
-            res_id = None
-            if ids:
-                res_id = ids[0]
-            self.screen.load(ids, modified=True)
-            self.screen.display(res_id=res_id)
-            if self.screen.current_view:
-                self.screen.current_view.set_cursor()
+        def callback(result):
+            if result:
+                ids = [x[0] for x in result]
+                self.screen.load(ids, modified=True)
+                self.screen.display(res_id=ids[0])
+            self.screen.set_cursor()
             self.wid_text.set_text('')
-
         if len(ids) != 1:
             WinSearch(model_name, callback, sel_multi=True,
-                ids=ids, context=context, domain=domain)
+                ids=ids, context=self.context, domain=domain)
         else:
-            callback(ids)
+            callback([(i, None) for i in ids])
 
     def _sig_label(self, screen, signal_data):
         name = '_'
-        if signal_data[0] >= 0:
+        access = common.MODELACCESS[screen.model_name]
+        readonly = screen.group.readonly
+        if signal_data[0] >= 1:
             name = str(signal_data[0])
+            if self.domain is not None:
+                self.but_remove.set_sensitive(True)
+            if signal_data[0] < signal_data[1]:
+                self.but_next.set_sensitive(True)
+            else:
+                self.but_next.set_sensitive(False)
+            if signal_data[0] > 1:
+                self.but_pre.set_sensitive(True)
+            else:
+                self.but_pre.set_sensitive(False)
+            if access['delete'] and not readonly:
+                self.but_del.set_sensitive(True)
+                self.but_undel.set_sensitive(True)
+        else:
+            self.but_del.set_sensitive(False)
+            self.but_undel.set_sensitive(False)
+            self.but_next.set_sensitive(False)
+            self.but_pre.set_sensitive(False)
+            if self.domain is not None:
+                self.but_remove.set_sensitive(False)
         line = '(%s/%s)' % (name, signal_data[1])
         self.label.set_text(line)
 
+    def activate_save(self, *args):
+        modified = self.screen.modified()
+        self.but_ok.props.sensitive = modified
+        self.win.set_default_response(
+            gtk.RESPONSE_OK if modified else gtk.RESPONSE_CANCEL)
+
+    def close(self, widget):
+        widget.emit_stop_by_name('close')
+        self.response(self.win, gtk.RESPONSE_CANCEL)
+        return True
+
     def response(self, win, response_id):
         validate = False
         cancel_responses = (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
         self.screen.current_view.set_value()
+        readonly = self.screen.group.readonly
         if (response_id not in cancel_responses
+                and not readonly
                 and self.screen.current_record is not None):
             validate = self.screen.current_record.validate(
                 self.screen.current_view.get_fields())
+            if validate and self.screen.pre_validate:
+                validate = self.screen.current_record.pre_validate()
+            if validate and self.save_current:
+                if not self.screen.save_current():
+                    validate = False
+            elif validate and self.screen.current_view.view_type == 'form':
+                view = self.screen.current_view
+                for widgets in view.widgets.itervalues():
+                    for widget in widgets:
+                        if (hasattr(widget, 'screen')
+                                and widget.screen.pre_validate):
+                            record = widget.screen.current_record
+                            if record:
+                                validate = record.pre_validate()
             if not validate:
-                self.screen.current_view.set_cursor()
+                self.screen.set_cursor()
                 self.screen.display()
                 return
             if response_id == gtk.RESPONSE_ACCEPT:
                 self.new()
                 return
-        if (self.but_cancel
-                and self.screen.current_record
+        if (self.screen.current_record
+                and not readonly
                 and response_id in cancel_responses):
-            self.screen.group.remove(self.screen.current_record, remove=True)
+            if (self.screen.current_record.id < 0
+                    or self.save_current):
+                self.screen.group.remove(self.screen.current_record,
+                    remove=True)
+            elif self.screen.current_record.modified:
+                self.screen.current_record.cancel()
+                self.screen.current_record.reload()
+                self.screen.current_record.signal('record-changed')
             result = False
         else:
             result = response_id not in cancel_responses
-        self.destroy()
         self.callback(result)
+        self.destroy()
 
     def new(self):
-        self.screen.new(context=self.context)
+        self.screen.new()
         self.screen.current_view.display()
-        self.screen.current_view.set_cursor(new=True)
+        self.screen.set_cursor(new=True)
+        self.many -= 1
+        if self.many == 0:
+            self.but_new.set_sensitive(False)
+            self.win.set_default_response(gtk.RESPONSE_OK)
 
     def destroy(self):
         self.screen.screen_container.alternate_view = False
